from unittest import TestCase

from dnslookup import dns


class TestLookup(TestCase):
    def test_lookup(self):
        result = dns.nslookup("google.com", "8.8.8.8")
        print(result)
        self.fail()



#!/usr/bin/env python
import subprocess as sp
import sys
import re
import time

nslookup_regex = re.compile(r"(Address):(\t| )?(\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})$")
ip_address = re.compile('(\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})')


def nslookup(domain, nameserver_ip):
    try:

        command = "nslookup"
        start = time.time()
        process = sp.Popen([command, domain, nameserver_ip], stdin=sp.PIPE, stdout=sp.PIPE, close_fds=True)
        end = time.time()

        (stdout, stdin, stderr) = (process.stdout, process.stdin, process.stderr)

        data = stdout.readlines()
        data = [line.decode('utf-8') for line in data]
        data = list(filter(nslookup_regex.match, data))
        ips = []

        for response in data:
            match = ip_address.search(response)
            if match:
                try:
                    ips.append(match.groups(1)[0])
                except IndexError:
                    pass # We don't care if we cant get some of the ips, but this really should happen
        return list(set(ips)), end - start

    except KeyboardInterrupt:
        print("Exiting...")
        sys.exit(1)


if __name__ == '__main__':
    result = nslookup("google.com", "8.8.8.8")
    print(result)



from dnslookup.DNSPythonBase import DNSPythonBase


class NameServer(DNSPythonBase):
    def __init__(self, ip, name, country_id, city=None, version=None, error=None, dnssec=None, reliability=None,
                 checked_at=None, created_at=None):
        self.ip = ip
        self.name = name
        self.country_id = country_id
        self.city = city
        self.version = version
        self.error = error
        self.dnssec = dnssec
        self.reliability = reliability
        self.checked_at = checked_at
        self.created_at = created_at

    @classmethod
    def BuildFromCSV(cls, csv_row):
        try:
            return NameServer(ip=csv_row['ip'], name=csv_row['name'], country_id=csv_row['country_id'],
                              city=csv_row['city'], version=csv_row['version'], error=csv_row['error'],
                              dnssec=csv_row['dnssec'], reliability=csv_row['reliability'],
                              checked_at=csv_row['checked_at'], created_at=csv_row['created_at'])
        except Exception:
            raise RuntimeWarning("Couldn't get the right headers out of the CSV when creating a NameServer "
                                 "Object\nMake sure that the supplied URL has the following headers:\n\tip,name,"
                                 "country_id,city,version,error,dnssec,reliability,checked_at,created_at")



from future.standard_library import install_aliases

from dnslookup import dns
from dnslookup.DNSPythonBase import DNSPythonBase

from queue import Queue


class DomainDNSRecord(DNSPythonBase):
    def __init__(self, name_server, records, response_time):
        self.name_server = name_server
        self.records = records
        self.response_time = response_time


class Domain(DNSPythonBase):
    def __init__(self, url):
        self.url = url
        self.results = Queue()
        self.status = Queue()

    def lookup(self, name_server_collection):
        for key, name_server in name_server_collection.collection.items():
            ips, time = dns.nslookup(domain=self.url, nameserver_ip=name_server.ip)
            self.results.put(DomainDNSRecord(name_server=name_server, records=ips, response_time=time))
        self.status.put("Finished")



#!/usr/bin/env python

import argparse
import csv
from future.standard_library import install_aliases
install_aliases()

from threading import Thread
from queue import Empty

from dnslookup.Domain import Domain
from dnslookup.NameServer import NameServer
from dnslookup.NameServerCollection import NameServerCollection
from dnslookup.dns import *



collection = None
urlArgument = 1
args = None
parse = None


def commandLineInt():
    global args
    global parser
    parser = argparse.ArgumentParser()
    parser.add_argument("--level", "-l", default="1",
                        help="This option allows you to: (1) use a large database of DNS Servers from public-dns.info "
                             "(2) Use just the google DNS Servers. (3) Use your own server list. <Requires properly "
                             "formatted file> ")
    parser.add_argument("--file", "-f",
                        help="If provided, it will output to the file instead of console. File must be a csv")

    parser.add_argument("domain", help="The domain that you would like to use. ")
    parser.add_argument("--url", help="URL to use with option (3) of level")
    args = parser.parse_args()

    if args.level == "3" and args.url is None:
        parser.print_help()
        sys.exit(-1)

    if args.file is not None:
        if not str(args.file).endswith("csv"):
            parser.print_help()
            sys.exit(-1)


def setup():
    global args
    global collection
    if args.level == "1":
        collection = NameServerCollection.CollectionFromCSVURL(url="https://public-dns.info/nameservers.csv")
    elif args.level == "2":
        collection = NameServerCollection()
        collection.add_server(NameServer(ip="8.8.8.8", name="Google 1", country_id="US"))
        collection.add_server(NameServer(ip="8.8.4.4", name="Google 2", country_id="US"))
    elif args.level == "3":
        collection = NameServerCollection.CollectionFromCSVURL(url=args.url)
    else:
        parser.print_help()
        sys.exit(-1)


def console(domain):
    print("Result for {0}".format(str(domain.url)))
    print("{country} {ip} {result}".format(country="Country".rjust(7),
                                           ip="DNS Server".rjust(24),
                                           result="Result".rjust(22)))
    while domain.status.empty():
        try:
            mapping = domain.results.get(block=True, timeout=2)
            if mapping is not None:
                for record in mapping.records:
                    print(
                        "{country} \t{ip} \t\t{Result}".format(country=str(mapping.name_server.country_id).rjust(7),
                                                               ip=str(mapping.name_server.ip).rjust(20),
                                                               Result=str(record).rjust(15)))
        except Empty:
            pass


def output_csv(domain):
    with open(args.file, 'w') as outputfile:
        writer = csv.writer(outputfile)
        writer.writerow(["Country", "DNS Server", "Result", "Response Timne"])
        while domain.status.empty():
            try:
                mapping = domain.results.get(block=True, timeout=2)
                if mapping is not None:
                    for record in mapping.records:
                        writer.writerow(
                            [mapping.name_server.country_id, mapping.name_server.ip, record, mapping.response_time])
            except Empty:
                pass


def main():
    commandLineInt()
    setup()
    domain = Domain(url=args.domain)
    t = Thread(target=domain.lookup, args=(collection,))
    t.daemon = True
    t.start()
    try:
        if args.file is None:
            console(domain=domain)
        else:
            output_csv(domain=domain)

    except KeyboardInterrupt:
        t.join(timeout=1)
        sys.exit(-1)
    t.join()


if __name__ == '__main__':
    main()



# encoding=utf8

from dnslookup.DNSPythonBase import DNSPythonBase
from dnslookup.NameServer import NameServer


class NameServerCollection(DNSPythonBase):
    def __init__(self):
        self.collection = {}

    def add_server(self, name_server):
        self.collection[name_server.ip] = name_server

    def lookup(self, domain):
        import dnslookup.dns as dns
        for name_server in self.collection:
            ips = dns.nslookup(domain=domain, nameserver_ip=name_server.ip)

    @classmethod
    def CollectionFromCSV(cls, csv_data):
        import csv
        collection = cls()
        reader = csv.DictReader(csv_data)
        try:
            for row in reader:
                collection.add_server(NameServer.BuildFromCSV(csv_row=row))
        except Exception as e:
            pass
        return collection

    @classmethod
    def CollectionFromCSVURL(cls, url):
        from future.standard_library import install_aliases
        install_aliases()
        from urllib.request import urlopen
        response = urlopen(url)
        # response = [str(line.decode('utf-8')) for line in response.readlines()]
        final = []
        for line in response.readlines():
            try:
                final.append(line.decode('utf-8').encode('utf-8'))
            except (UnicodeDecodeError, UnicodeEncodeError) as e:
                pass  # We cant decode something
        return cls.CollectionFromCSV(csv_data=final)



from distutils.core import setup

import setuptools


setup(
    name='dnslookup',
    version='0.7.30',
    author='Joubin Jabbari',
    author_email='joubin.j@gmail.com',
    url="https://github.com/joubin/DNSPython",
    license='LICENSE.txt',
      entry_points={
          'console_scripts':
              ['dnslookup= dnslookup.main:main']},
    description='Check dns entries across the world. In short, if you move your server from one IP to another, you can check to see when most of the dns servers in the world have update their results.',
    long_description="Test",
    long_description_content_type="text/markdown",
    packages=['dnslookup'],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ], install_requires=['future']
)
